from sympy import Li
from isostatic import LinealLoad, GeneralLoadLineal
from sympy.abc import E, I 

def test_linealLoads_angles():
    Q = LinealLoad(100,100,10)
    Qleft_angle =  (12500/3)/(E*I)
    Qright_angle = Qleft_angle
    assert Q.angleLeft == Qleft_angle
    assert Q.angleLeft == Q.angleRight
    assert Q.angleRight == Qright_angle

    R = LinealLoad(0,25,6)
    Rright_angle = 105.0/(E*I)
    Rleft_angle = 120.0/(E*I)

    assert R.angleLeft == Rleft_angle
    assert R.angleRight == Rright_angle

def test_generalLoads_to_constantLoads():
    Q = LinealLoad(100,100,10)
    R = GeneralLoadLineal(100, 100, 0, 10, 0)

    assert Q.angleLeft == R.angleLeft
    assert Q.angleRight == R.angleRight
    

    A = LinealLoad(0,25,6)
    B = GeneralLoadLineal(0, 25, 0, 6, 0)
    assert A.angleLeft == B.angleLeft
    assert A.angleRight == B.angleRight





class Point:
    '''
    A point is a position of a place in the space.
    But Isostatic only supports two-dimensions.

    '''
    def __init__(self, name, *args) -> None:
        if not isinstance(name, str):
            raise TypeError('name tiene que ser string')
        self.__args = [*args]
        self.__name = name

    def __str__(self) -> str:
        return (r'{type}: {coord}'.format(type = 'Point',coord = self.__args))

    def __repr__(self) -> str:
        return (r'{type}: {coord}'.format(type = 'Point',coord = self.__args))

    def __len__(self) -> int:
        return len(self.__args)

    def components(self):
        pass

    @property
    def coordinates(self):
        return self.__args

    @property
    def name(self):
        return self.__name
    
    @name.setter
    def name(self, name):
        if not isinstance(name, str):
            raise (TypeError('name tiene que ser string'))
        
        self.__name = name

    
        



from sympy import Symbol

modE = Symbol('E')
I_ = Symbol('I')

class LinealLoad:
    def __init__(self, q1: float, q2: float, L: float, E = modE, I = I_) -> None:
        self._q1 = q1
        self._q2 = q2
        self._L = L
        self._k = 1/(E*I)
        self._angleRight = self._k * (((self._L**3)*(8*self._q1 + 7*self._q2))/(360))
        self._angleLeft = self._k * (((self._L**3)*(7*self._q1 + 8*self._q2))/(360))
    
    @property
    def angleRight(self):
        return self._angleRight
    
    @property
    def angleLeft(self):
        return self._angleLeft

class GeneralLoadLineal:
    '''
    Este metodo es un caso general que se puede desplegar a 20 casos. 
    siendo:
    a: Longitud inicial desde el apoyo izquierdo hasta el inicio de la carga
    b: Longitud de la carga en la viga
    c: Longitud final desde el final de la carga hasta el apoyo derecho
    q1: valor de la carga en el lado inicial
    q2: valor de la carga en el lado final
    
    OPCIONAL:
    importar las constantes modE y I_ para implementar distintos valores algebraicamente
    '''
    def __init__(self, q1: float, q2: float, a: float, b: float, c: float, E = modE, I = I_) -> None:
        self._L = a + b + c
        self._a = a
        self._b = b
        self._c = c
        self._q1 = q1
        self._q2 = q2
        self._k = 1/(modE*I_)

        self._angleRight = self._k*((self._L**3)/360)*(self._q1*((10*((self._b*(3*self._c+2*self._b))/(self._L**2)))-(15*((self._b+self._c)/self._L)**4)+(3*((((self._b + self._c)**5)-self._c**5)/(self._b * self._L**4)))) + self._q2*((10*((self._b*(3*self._c + self._b))/(self._L**2)))+(15*(self._c/self._L)**4)-(3*((((self._b + self._c)**5)-self._c**5)/(self._b * self._L**4)))) )

        self._angleLeft = self._k*((self._L**3)/360)*(self._q2*((10*((self._b*(3*self._a+2*self._b))/(self._L**2)))-(15*((self._a+self._b)/self._L)**4)+(3*((((self._a + self._b)**5)-self._a**5)/(self._b * self._L**4)))) + self._q1*((10*((self._b*(3*self._a + self._b))/(self._L**2)))+(15*(self._a/self._L)**4)-(3*((((self._a + self._b)**5)-self._a**5)/(self._b * self._L**4)))) )

    @property
    def angleRight(self):
        return self._angleRight
    
    @property
    def angleLeft(self):
        return self._angleLeft

        




from isostatic import Support, Point

def test_istances():
    A = Point('A', 0, 0)
    RA = Support(A, vertical=True)
    assert isinstance(RA, Support)

    '''
    Errores en los tipos proporcionados(RESUELTO)
    '''


from sympy import Symbol
from ..points.point import Point

class Support:
    '''
    A support is a element in beams that limited a movement in a axis 
    '''
    def __init__(self, pointImplementation, horizontal = False, vertical = False, moment = False, rotate_degrees = 0) -> None:
        if not isinstance(pointImplementation, Point):
            raise TypeError('Point necesariamente debe ser class Point')
        if horizontal:
            self.__nameH = 'hor_{}'.format(pointImplementation.name)
            self.__H = Symbol(self.__nameH)        
        if vertical:
            self.__nameV = 'vert_{}'.format(pointImplementation.name)
            self.__V = Symbol(self.__nameV)     
        if moment:
            self.__nameM = 'mom_{}'.format(pointImplementation.name)
            self.__M = Symbol(self.__nameM)           
        
        self.__rotate_degrees = rotate_degrees
        self.__coord = pointImplementation.coordinates
    

        
        
        


from isostatic import Point
from pytest import raises

def test_str_point():
    str_A = "Point: [4, 5, 6]"
    A = Point('A', 4, 5, 6)
    assert str(A) == str_A

    str_B = "Point: []"
    B = Point('B')
    assert str(B) == str_B

    str_C = "Point: [2]"
    C = Point('C',2)
    assert str_C == str(C)

    str_D = "Point: [1, 5, 8, 7, 9]"
    D = Point('D',1, 5, 8, 7, 9)
    assert str(D) == str_D



def test_len_point():
    assert len(Point('A',4,8,2,3,-1,4,2))== 7
    assert len(Point('B',1,2,3)) == 3
    assert len(Point('C',100)) == 1
    assert len(Point('D')) == 0


def test_raises():
    with raises(TypeError):
        Point(5,3,1,2)
    with raises(TypeError):
        Point(True,3,1,2)
    with raises(TypeError):
        Point(False,3,1,2)

    A = Point('Z', 4,5)
    with raises(TypeError):
        A.name = A
    
def test_getters():
    A = Point('A', 1, 2, 3)
    assert A.name == 'A'


def test_setters():
    B = Point('Z', 4, 5, 6)
    B.name = 'B'
    assert  B.name == 'B'
